@@ -33,15 +33,15 @@
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/DataStream.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 #include <Lum/Model/String.h>
 #include <Lum/Model/Table.h>
 
 #include <Lum/OS/Color.h>
-#include <Lum/OS/Main.h>
 
-#include <Lum/OS/Manager/Repository.h>
+#include <Lum/Manager/Display.h>
 
+#include <Lum/Application.h>
 #include <Lum/Label.h>
 #include <Lum/PercentPie.h>
 #include <Lum/Table.h>
@@ -109,65 +109,43 @@ public:
 typedef Lum::Model::StdTable<DiskInfo>      DiskInfoModel;
 typedef Lum::Base::Reference<DiskInfoModel> DiskInfoModelRef;
 
-class DiskInfoObject : public Lum::Object
+class DiskInfoModelPainter : public Lum::TableCellPainter
 {
 private:
-  DiskInfo          diskInfo;
-  Lum::OS::FontRef  font;
   Lum::OS::ColorRef red;
   Lum::OS::ColorRef green;
 
 public:
-  DiskInfoObject()
-  : font(Lum::OS::display->GetFont()),
-    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+  DiskInfoModelPainter()
+  : red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
     green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
   {
     // no code
   }
 
-  void SetDiskInfo(const DiskInfo& diskInfo)
+  size_t GetProposedHeight() const
   {
-    this->diskInfo=diskInfo;
+    return Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+           GetFont()->height+
+           GetFont()->height+
+           Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
   }
 
-  void CalcSize()
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    minWidth=10*font->StringWidth(L"m",Lum::OS::Font::normal);
-    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-              font->height+
-              font->height+
-              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
+    DiskInfo     diskInfo=dynamic_cast<const DiskInfoModel*>(GetModel())->GetEntry(GetRow());
     std::wstring sizeText=Lum::Base::ByteSizeToWString(diskInfo.avail*diskInfo.unit*1024)+L" free";
-    size_t       sizeTextWidth=font->StringWidth(sizeText);
+    size_t       sizeTextWidth=GetFont()->StringWidth(sizeText);
 
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
-    draw->DrawString(this->x,
-                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+font->ascent,
+    draw->PushFont(GetFont());
+    draw->PushForeground(GetTextColor(draw));
+    draw->DrawString(x,
+                     y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+GetFont()->ascent,
                      diskInfo.dir);
 
-    draw->DrawString(this->x+width-sizeTextWidth+1,
-                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+font->ascent,
+    draw->DrawString(x+width-sizeTextWidth+1,
+                     y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+GetFont()->ascent,
                      sizeText);
 
     draw->PopForeground();
@@ -183,47 +161,17 @@ public:
       draw->PushForeground(green);
     }
 
-    draw->FillRectangle(this->x,
-                        this->y+
+    draw->FillRectangle(x,
+                        y+
                         Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
-                        font->height,
+                        GetFont()->height,
                         (size_t)(width*percent),
-                        font->height);
+                        GetFont()->height);
 
     draw->PopForeground();
   }
 };
 
-class DiskInfoDataProvider : public DiskInfoModel::DataProvider
-{
-private:
-  DiskInfoObject *object;
-
-public:
-  DiskInfoDataProvider()
-  : object(new DiskInfoObject())
-  {
-    object->SetFlex(true,true);
-    object->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
-  }
-
-  virtual ~DiskInfoDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const DiskInfoModel::Iterator& iter, size_t column) const
-  {
-    switch (column) {
-    case 1:
-      object->SetDiskInfo(*iter);
-      return object;
-    default:
-      return NULL;
-    }
-  }
-};
-
 class MainWindow : public Lum::Dialog
 {
 private:
@@ -256,7 +204,7 @@ public:
     refreshTime(new Lum::Model::Int(15)),
     timer(new Lum::Model::Action()),
     selection(new Lum::Model::SingleLineSelection),
-    tableModel(new DiskInfoModel(new DiskInfoDataProvider())),
+    tableModel(new DiskInfoModel()),
     device(new Lum::Model::String()),
     dir(new Lum::Model::String()),
     used(new Lum::Model::String()),
@@ -321,6 +269,7 @@ public:
     table->SetSelection(selection);
     table->SetDoubleClickAction(showMap);
     table->SetShowHeader(false);
+    table->SetPainter(new DiskInfoModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
     panel->Add(table);
 
@@ -482,8 +431,8 @@ public:
     if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
       diskInfoThread.Start();
 
-      if (Lum::OS::Manager::repository->GetDisplayManager()!=NULL) {
-        Observe(Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayStateChangeAction());
+      if (Lum::Manager::Display::Instance()!=NULL) {
+        Observe(Lum::Manager::Display::Instance()->GetDisplayStateChangeAction());
       }
     }
     else if (model==diskInfoThread.dataAvailable &&
@@ -496,9 +445,9 @@ public:
         selection->SelectLine(1);
       }
 
-      if (Lum::OS::Manager::repository->GetDisplayManager()==NULL ||
-          (Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayState()==Lum::OS::Manager::DisplayManager::DisplayStateOn ||
-           Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayState()==Lum::OS::Manager::DisplayManager::DisplayStateUnknown)) {
+      if (Lum::Manager::Display::Instance()==NULL ||
+          (Lum::Manager::Display::Instance()->GetDisplayState()==Lum::Manager::Display::DisplayStateOn ||
+           Lum::Manager::Display::Instance()->GetDisplayState()==Lum::Manager::Display::DisplayStateUnknown)) {
         Lum::OS::display->AddTimer(refreshTime->Get(),0,timer);
       }
     }
@@ -525,11 +474,11 @@ public:
     else if (model==about && about->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
     }
-    else if (Lum::OS::Manager::repository->GetDisplayManager()!=NULL &&
-             Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayStateChangeAction()==model &&
-             Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayStateChangeAction()->IsFinished()) {
-      if (Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayState()==Lum::OS::Manager::DisplayManager::DisplayStateOn ||
-        Lum::OS::Manager::repository->GetDisplayManager()->GetDisplayState()==Lum::OS::Manager::DisplayManager::DisplayStateUnknown) {
+    else if (Lum::Manager::Display::Instance()!=NULL &&
+             Lum::Manager::Display::Instance()->GetDisplayStateChangeAction()==model &&
+             Lum::Manager::Display::Instance()->GetDisplayStateChangeAction()->IsFinished()) {
+      if (Lum::Manager::Display::Instance()->GetDisplayState()==Lum::Manager::Display::DisplayStateOn ||
+        Lum::Manager::Display::Instance()->GetDisplayState()==Lum::Manager::Display::DisplayStateUnknown) {
         Lum::OS::display->RemoveTimer(timer);
         Lum::OS::display->AddTimer(refreshTime->Get(),0,timer);
       }
@@ -540,10 +489,10 @@ public:
 
 };
 
-class Main : public Lum::OS::MainDialog<MainWindow>
+class Main : public Lum::GUIApplication<MainWindow>
 {
 public:
-  bool Prepare()
+  bool Initialize()
   {
 #if defined(APP_DATADIR)
     Lum::Base::Path::SetApplicationDataDir(Lum::Base::StringToWString(APP_DATADIR));
@@ -557,7 +506,7 @@ public:
     info.SetCopyright(L"(c) 2004, Tim Teulings");
     info.SetLicense(L"GNU Public License");
 
-    return Lum::OS::MainDialog<MainWindow>::Prepare();
+    return Lum::GUIApplication<MainWindow>::Initialize();
   }
 };
 
